@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25952 $
+ * $Revision: 28134 $
  * $Author: rjongbloed $
- * $Date: 2011-06-01 01:02:58 -0500 (Wed, 01 Jun 2011) $
+ * $Date: 2012-08-01 19:48:35 -0500 (Wed, 01 Aug 2012) $
  */
 
 #ifndef OPAL_CODEC_OPALPLUGINMGR_H
@@ -119,6 +119,8 @@ class OpalPluginCodecManager : public PPluginModuleManager
     OpalPluginCodecManager(PPluginManager * pluginMgr = NULL);
     ~OpalPluginCodecManager();
 
+    PFACTORY_GET_SINGLETON(PFactory<PPluginModuleManager>, OpalPluginCodecManager);
+
     void RegisterStaticCodec(const H323StaticPluginCodecFactory::Key_T & name,
                              PluginCodec_GetAPIVersionFunction getApiVerFn,
                              PluginCodec_GetCodecFunction getCodecFn);
@@ -326,7 +328,10 @@ class OpalPluginVideoTranscoder : public OpalVideoTranscoder, public OpalPluginT
     bool DecodeFrame(const RTP_DataFrame & src, RTP_DataFrameList & dstList);
 
     RTP_DataFrame * m_bufferRTP;
+    DWORD           m_lastSequenceNumber;
     DWORD           m_lastDecodedTimestamp; // For missing marker bit detection
+    DWORD           m_lastMarkerTimestamp;  // For continuous marker bit detection
+    bool            m_badMarkers;
     PSimpleTimer    m_videoPictureLossTimer;
 
 #if PTRACING
@@ -338,7 +343,7 @@ class OpalPluginVideoTranscoder : public OpalVideoTranscoder, public OpalPluginT
 
 ///////////////////////////////////////////////////////////////////////////////
 
-#if OPAL_T38_CAPABILITY
+#if OPAL_FAX
 
 class OpalPluginFaxFormatInternal : public OpalMediaFormatInternal, public OpalPluginMediaFormatInternal
 {
@@ -355,7 +360,7 @@ class OpalPluginFaxFormatInternal : public OpalMediaFormatInternal, public OpalP
     virtual bool IsValidForProtocol(const PString & protocol) const;
 };
 
-#endif // OPAL_T38_CAPABILITY
+#endif // OPAL_FAX
 
 
 //////////////////////////////////////////////////////
@@ -569,33 +574,6 @@ class H323CodecPluginGenericAudioCapability : public H323GenericAudioCapability,
 
 #if OPAL_VIDEO
 
-//////////////////////////////////////////////////////////////////////////////
-//
-// Class for handling most audio plugin capabilities
-//
-
-class H323VideoPluginCapability : public H323VideoCapability,
-                             public H323PluginCapabilityInfo
-{
-  PCLASSINFO(H323VideoPluginCapability, H323VideoCapability);
-  public:
-    H323VideoPluginCapability(const PluginCodec_Definition * codecDefn,
-                              const OpalMediaFormat & mediaFormat,
-                              unsigned _pluginSubType);
-
-    virtual PString GetFormatName() const;
-
-    virtual unsigned GetSubType() const;
-
-    static bool SetOptionsFromMPI(OpalMediaFormat & mediaFormat, int frameWidth, int frameHeight, int frameRate);
-
-    virtual void PrintOn(std::ostream & strm) const;
-
-  protected:
-    unsigned pluginSubType;
-    unsigned h323subType;   // only set if using capability without codec
-};
-
 //////////////////////////////////////////////////////////////////////////////
 //
 // Class for handling non standard video capabilities
@@ -644,9 +622,10 @@ class H323CodecPluginGenericVideoCapability : public H323GenericVideoCapability,
 // Class for handling H.261 plugin capabilities
 //
 
-class H323H261PluginCapability : public H323VideoPluginCapability
+class H323H261PluginCapability : public H323VideoCapability,
+                                 public H323PluginCapabilityInfo
 {
-  PCLASSINFO(H323H261PluginCapability, H323VideoPluginCapability);
+    PCLASSINFO(H323H261PluginCapability, H323VideoCapability);
   public:
     H323H261PluginCapability(const PluginCodec_Definition * codecDefn, const OpalMediaFormat & mediaFormat);
 
@@ -654,6 +633,9 @@ class H323H261PluginCapability : public H323VideoPluginCapability
 
     virtual PObject * Clone() const;
 
+    virtual PString GetFormatName() const;
+    virtual unsigned GetSubType() const;
+
     virtual PBoolean OnSendingPDU(
       H245_VideoCapability & pdu  /// PDU to set information on
     ) const;
@@ -672,9 +654,10 @@ class H323H261PluginCapability : public H323VideoPluginCapability
 // Class for handling H.263 plugin capabilities
 //
 
-class H323H263PluginCapability : public H323VideoPluginCapability
+class H323H263PluginCapability : public H323VideoCapability,
+                                 public H323PluginCapabilityInfo
 {
-  PCLASSINFO(H323H263PluginCapability, H323VideoPluginCapability);
+  PCLASSINFO(H323H263PluginCapability, H323VideoCapability);
   public:
     H323H263PluginCapability(const PluginCodec_Definition * codecDefn, const OpalMediaFormat & mediaFormat);
 
@@ -682,6 +665,9 @@ class H323H263PluginCapability : public H323VideoPluginCapability
 
     virtual PObject * Clone() const;
 
+    virtual PString GetFormatName() const;
+    virtual unsigned GetSubType() const;
+
     virtual PBoolean OnSendingPDU(
       H245_VideoCapability & pdu  /// PDU to set information on
     ) const;
@@ -693,7 +679,10 @@ class H323H263PluginCapability : public H323VideoPluginCapability
     virtual PBoolean OnReceivedPDU(
       const H245_VideoCapability & pdu  /// PDU to get information from
     );
-    virtual PBoolean IsMatch(const PASN_Choice & subTypePDU) const;
+    virtual PBoolean IsMatch(
+      const PASN_Choice & subTypePDU,     ///<  sub-type PDU of H323Capability
+      const PString & mediaPacketization  ///< Media packetization used
+    ) const;
 };
 
 #endif // OPAL_VIDEO
