@@ -27,9 +27,9 @@
  *     US Joint Forces Command Joint Concept Development & Experimentation (J9)
  *     http://www.jfcom.mil/about/abt_j9.htm
  *
- * $Revision: 25556 $
+ * $Revision: 27984 $
  * $Author: rjongbloed $
- * $Date: 2011-04-13 04:35:30 -0500 (Wed, 13 Apr 2011) $
+ * $Date: 2012-07-10 03:24:24 -0500 (Tue, 10 Jul 2012) $
  */
 
 #ifndef OPAL_OPAL_CONNECTION_H
@@ -73,10 +73,12 @@ class PURL;
 #define OPAL_OPT_AUTO_START           "AutoStart"             ///< String option for auto-started media types
 #define OPAL_OPT_CALL_IDENTIFIER      "Call-Identifier"       ///< String option to override generated call identifier
 #define OPAL_OPT_CALLING_PARTY_URL    "Calling-Party-URL"     ///< String option to set outgoing local URL
-#define OPAL_OPT_CALLING_PARTY_NUMBER "Calling-Party-Number"  ///< String option to set outgoing local number
+#define OPAL_OPT_CALLING_PARTY_NUMBER "Calling-Party-Number"  ///< String option to set outgoing local number, deprecated, same as OPAL_OPT_CALLING_PARTY_NAME
 #define OPAL_OPT_CALLING_PARTY_NAME   "Calling-Party-Name"    ///< String option to set outgoing local name
 #define OPAL_OPT_CALLING_PARTY_DOMAIN "Calling-Party-Domain"  ///< String option to set outgoing local host/address/domain
 #define OPAL_OPT_CALLING_DISPLAY_NAME "Calling-Display-Name"  ///< String option to set outgoing display name
+#define OPAL_OPT_CALLED_PARTY_NAME    "Called-Party-Name"     ///< String option to set outgoing party name
+#define OPAL_OPT_CALLED_DISPLAY_NAME  "Called-Display-Name"   ///< String option to set outgoing display name
 #define OPAL_OPT_REDIRECTING_PARTY    "Redirecting-Party"     ///< String option to set redirecting/refferred-by party
 #define OPAL_OPT_PRESENTATION_BLOCK   "Presentation-Block"    ///< String option to block outgoing calling number presentation
 #define OPAL_OPT_ORIGINATOR_ADDRESS   "Originator-Address"    ///< String option for originator address used by IVR
@@ -584,12 +586,17 @@ class OpalConnection : public PSafeObject
 
        Note that this function will return quickly as the release and
        disposal of the connections is done by another thread.
+
+       If \p sync is not NULL then it is signalled when the calls are cleared.
       */
     void ClearCall(
-      CallEndReason reason = EndedByLocalUser ///<  Reason for call clearing
+      CallEndReason reason = EndedByLocalUser, ///<  Reason for call clearing
+      PSyncPoint * sync = NULL
     );
 
     /**Clear a current connection, synchronously.
+       This hangs up the connection to a remote endpoint. Note that this function
+       is always synchronous. If \p sync is NULL then a local PSyncPoint is used.
       */
     virtual void ClearCallSynchronous(
       PSyncPoint * sync,                       ///<  Synchronisation object to signal
@@ -810,34 +817,54 @@ class OpalConnection : public PSafeObject
     /**A call back function to monitor the progress of a transfer.
        When a transfer operation is initiated, the Transfer() function will
        generally return immediately and the transfer may take some time. This
-       call back can give indication to the application of the progress of
+       call back can give an indication to the application of the progress of
+       the transfer.
        the transfer.
 
-       For example in SIP, the Transfer() function will have sent a REFER
-       request to the remote party. The remote party sends us NOTIFY requests
-       about the progress of the REFER request.
+       For example in SIP, the OpalCall::Transfer() function will have sent a
+       REFER request to the remote party. The remote party sends us NOTIFY
+       requests about the progress of the REFER request.
 
        An application can now make a decision during the transfer operation
        to short circuit the sequence, or let it continue. It can also
        determine if the transfer did not go through, and it should "take back"
-       the call.
+       the call. Note no action is required to "take back" the call other than
+       indicate to the user that they are back on.
 
        A return value of false will immediately disconnect the current call.
 
        The exact format of the \p info parameter is dependent on the protocol
-       being used. It will always have a value info["result"] which will be
-       "success" or "failed". Other variables are protocol dependent.
+       being used. As a minimum, it will always have a values info["result"]
+       and info["party"].
 
-       For SIP, there is an additional info["state"] containing the NOTIFY
-       subscription state and an info["code"] entry containing the 3 digit
-       code returned in the NOTIFY body.
+       The info["party"] indicates the part the \p connection is playing in
+       the transfer. This will be:
+          "A"   party being transferred
+          "B"   party initiating the transfer of "A"
+          "C"   party "A" is being transferred to
+
+       The info["result"] will be at least one of the following:
+          "success"     Transfer completed successfully (party A or B)
+          "incoming"    New call was from a transfer (party C)
+          "started"     Transfer operation has started (party A)
+          "progress"    Transfer is in progress (party B)
+          "blind"       Transfer is blind, no further notification (party B)
+          "error"       Transfer could not begin (party B)
+          "failed"      Transfer started but did not complete (party A or B)
+
+       For SIP, there may be an additional info["state"] containing the NOTIFY
+       subscription state, an info["code"] entry containing the 3 digit
+       code returned in the NOTIFY body and info["Referred-By"] indicating the
+       URI of party B. Other fields may also be present.
 
        The default behaviour calls the OpalEndPoint function of the same name.
        The default action of that function is to return false, thereby
        releasing the connection if the info["result"] == "success".
      */
     virtual bool OnTransferNotify(
-      const PStringToString & info ///< State of the transfer
+      const PStringToString & info, ///< State of the transfer
+      const OpalConnection * transferringConnection
+                                    ///< Connection performing transfer, maybe "this"
     );
 
     /**Release the current connection.
@@ -849,7 +876,8 @@ class OpalConnection : public PSafeObject
        disposal of the connections is done by another thread.
       */
     virtual void Release(
-      CallEndReason reason = EndedByLocalUser ///<  Reason for call release
+      CallEndReason reason = EndedByLocalUser, ///<  Reason for call release
+      bool synchronous = false                 ///< If true spawns thread to release connection
     );
 
     /**Clean up the termination of the connection.
@@ -1141,10 +1169,15 @@ class OpalConnection : public PSafeObject
       OpalMediaPatch & patch    ///< Patch being stopped
     );
 
-    /** Notifier function for OpalVideoUpdatePicture.
+    /** Callback for media commands.
         Calls the SendIntraFrameRequest on the rtp session
+
+       @returns true if command is handled.
       */
-    PDECLARE_NOTIFIER(OpalMediaCommand, OpalConnection, OnMediaCommand);
+    virtual bool OnMediaCommand(
+      OpalMediaStream & stream,         ///< Stream command executed on
+      const OpalMediaCommand & command  ///< Media command being executed
+    );
 
     /**Attaches the RFC 2833 handler to the media patch
        This method may be called from subclasses, e.g. within
@@ -1739,7 +1772,6 @@ class OpalConnection : public PSafeObject
     PString              remotePartyAddress;
     PString              m_redirectingParty;
     CallEndReason        callEndReason;
-    bool                 synchronousOnRelease;
     PString              m_calledPartyNumber;
     PString              m_calledPartyName;
 
@@ -1860,6 +1892,7 @@ class OpalConnection : public PSafeObject
     P_REMOVE_VIRTUAL(bool, RetrieveConnection(), false);
     P_REMOVE_VIRTUAL(bool, IsConnectionOnHold(bool), false);
     P_REMOVE_VIRTUAL_VOID(ApplyStringOptions(OpalConnection::StringOptions &));
+    P_REMOVE_VIRTUAL(bool, OnTransferNotify(const PStringToString &), false);
 };
 
 #endif // OPAL_OPAL_CONNECTION_H
