@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25422 $
+ * $Revision: 27984 $
  * $Author: rjongbloed $
- * $Date: 2011-03-28 17:11:56 -0500 (Mon, 28 Mar 2011) $
+ * $Date: 2012-07-10 03:24:24 -0500 (Tue, 10 Jul 2012) $
  */
 
 #ifndef OPAL_OPAL_ENDPOINT_H
@@ -110,6 +110,12 @@ class OpalEndPoint : public PObject
     /**Add a set of listeners to the endoint.
        This allows for the automatic creating of incoming call connections. 
        If the list is empty then GetDefaultListeners() is used.
+
+       Note: while the \p interfaces parameter is a string array, each element
+       of the array should be compatible with OpalTransportAddress.
+
+       See OpalTransportAddress for more details on the syntax of an interface
+       definition.
       */
     PBoolean StartListeners(
       const PStringArray & interfaces ///<  Address of interface to listen on.
@@ -118,6 +124,9 @@ class OpalEndPoint : public PObject
     /**Add a listener to the endoint.
        This allows for the automatic creating of incoming call connections. /
        If the address is empty then the first entry of GetDefaultListeners() is used.
+
+       See OpalTransportAddress for more details on the syntax of an interface
+       definition.
       */
     PBoolean StartListener(
       const OpalTransportAddress & iface ///<  Address of interface to listen on.
@@ -412,19 +421,46 @@ class OpalEndPoint : public PObject
     /**A call back function to monitor the progress of a transfer.
        When a transfer operation is initiated, the Transfer() function will
        generally return immediately and the transfer may take some time. This
-       call back can give indicateion to the application of the progress of
+       call back can give an indication to the application of the progress of
+       the transfer.
        the transfer.
 
-       The exact format of the \p info parameter is dependent on the protocol
-       being used. It will always have a value info["result"] which will be
-       "success" or "failed". Other variables are protocol dependent.
+       For example in SIP, the OpalCall::Transfer() function will have sent a
+       REFER request to the remote party. The remote party sends us NOTIFY
+       requests about the progress of the REFER request.
 
-       For SIP, there is an additional info["state"] containing the NOTIFY
-       subscription state and an info["code"] entry containing the 3 digit
-       code returned in the NOTIFY body.
+       An application can now make a decision during the transfer operation
+       to short circuit the sequence, or let it continue. It can also
+       determine if the transfer did not go through, and it should "take back"
+       the call. Note no action is required to "take back" the call other than
+       indicate to the user that they are back on.
 
        A return value of false will immediately disconnect the current call.
 
+       The exact format of the \p info parameter is dependent on the protocol
+       being used. As a minimum, it will always have a values info["result"]
+       and info["party"].
+
+       The info["party"] indicates the part the \p connection is playing in
+       the transfer. This will be:
+          "A"   party being transferred
+          "B"   party initiating the transfer of "A"
+          "C"   party "A" is being transferred to
+
+       The info["result"] will be at least one of the following:
+          "success"     Transfer completed successfully (party A or B)
+          "incoming"    New call was from a transfer (party C)
+          "started"     Transfer operation has started (party A)
+          "progress"    Transfer is in progress (party B)
+          "blind"       Transfer is blind, no further notification (party B)
+          "error"       Transfer could not begin (party B)
+          "failed"      Transfer started but did not complete (party A or B)
+
+       For SIP, there may be an additional info["state"] containing the NOTIFY
+       subscription state, an info["code"] entry containing the 3 digit
+       code returned in the NOTIFY body and info["Referred-By"] indicating the
+       URI of party B. Other fields may also be present.
+
        The default behaviour calls the OpalManager function of the same name.
        The default action of that function is to return false, thereby
        releasing the connection if the info["result"] == "success".
@@ -440,6 +476,8 @@ class OpalEndPoint : public PObject
        call disposed of. Note that this function returns quickly and the
        disposal happens at some later time by a background thread. This it is
        safe to call this function from anywhere.
+
+       If \p sync is not NULL then it is signalled when the calls are cleared.
       */
     virtual PBoolean ClearCall(
       const PString & token,    ///<  Token for identifying connection
@@ -448,8 +486,8 @@ class OpalEndPoint : public PObject
     );
 
     /**Clear a current connection.
-       This hangs up the connection to a remote endpoint. Note that these functions
-       are synchronous
+       This hangs up the connection to a remote endpoint. Note that this function
+       is always synchronous. If \p sync is NULL then a local PSyncPoint is used.
       */
     virtual PBoolean ClearCallSynchronous(
       const PString & token,    ///<  Token for identifying connection
@@ -657,7 +695,15 @@ class OpalEndPoint : public PObject
 
   /**@name Other services */
   //@{
-    /**Callback called when Message Waiting Indication (MWI) is received
+    /**Callback called when Message Waiting Indication (MWI) is received.
+       Multiple callbacks may occur with each MessageWaitingType. A \p type
+       of NumMessageWaitingTypes indicates the server is unable to distinguish
+       the message type.
+
+       The \p extraInfo parameter is generally of the form "a/b" where a and b
+       unsigned integers representing new and old message count. However, it
+       may be a simple "yes" or "no" if the remote cannot provide a message
+       count.
      */
     virtual void OnMWIReceived (
       const PString & party,                ///< Name of party MWI is for
@@ -782,7 +828,7 @@ class OpalEndPoint : public PObject
     PMutex inUseFlag;
 
     friend void OpalManager::GarbageCollection();
-    friend void OpalConnection::Release(CallEndReason reason);
+    friend void OpalConnection::Release(CallEndReason,bool);
 
   private:
     P_REMOVE_VIRTUAL(PBoolean, OnIncomingConnection(OpalConnection &, unsigned), false);
