@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 26222 $
+ * $Revision: 28005 $
  * $Author: rjongbloed $
- * $Date: 2011-07-18 04:52:34 -0500 (Mon, 18 Jul 2011) $
+ * $Date: 2012-07-12 02:32:48 -0500 (Thu, 12 Jul 2012) $
  */
 
 #ifndef OPAL_SIP_SIPCON_H
@@ -246,9 +246,9 @@ class SIPConnection : public OpalRTPConnection
        the REFER to the remote endpoint of this call to do an INVITE with
        Replaces header to the remote party of the supplied tokens call. This
        is used for consultation transfer where A calls B, B holds A, B calls C,
-       B transfers C to A. The last step is a REFER to C with call details of
-       A that are extracted from the A to B call leg. This short cut is
-       possible because A nd C may be other endpoints but both B's are in tis
+       B transfers A to C. The last step is a REFER to A with call details of
+       C that are extracted from the B to C call leg. This short cut is
+       possible because A nd C may be other endpoints but both B's are in this
        instance of OPAL.
        
        In the end, both calls are cleared. The OnTransferNotify() function can
@@ -584,6 +584,10 @@ class SIPConnection : public OpalRTPConnection
       */
     PRACKMode GetPRACKMode() const { return m_prackMode; }
 
+    /** Return a bit mask of the allowed SIP methods.
+      */
+    virtual unsigned GetAllowedMethods() const;
+
 #if OPAL_VIDEO
     /**Call when SIP INFO of type application/media_control+xml is received.
 
@@ -593,7 +597,15 @@ class SIPConnection : public OpalRTPConnection
     virtual PBoolean OnMediaControlXML(SIP_PDU & pdu);
 #endif
 
-    virtual void OnMediaCommand(OpalMediaCommand & note, INT extra);
+    /** Callback for media commands.
+        Calls the SendIntraFrameRequest on the rtp session
+
+       @returns true if command is handled.
+      */
+    virtual bool OnMediaCommand(
+      OpalMediaStream & stream,         ///< Stream command executed on
+      const OpalMediaCommand & command  ///< Media command being executed
+    );
 
     virtual void OnStartTransaction(SIPTransaction & transaction);
 
@@ -613,13 +625,16 @@ class SIPConnection : public OpalRTPConnection
 
   protected:
     virtual bool GarbageCollection();
+    void OnUserInputInlineRFC2833(OpalRFC2833Info & info, INT type);
+
     PDECLARE_NOTIFIER(PTimer, SIPConnection, OnSessionTimeout);
     PDECLARE_NOTIFIER(PTimer, SIPConnection, OnInviteResponseRetry);
     PDECLARE_NOTIFIER(PTimer, SIPConnection, OnInviteResponseTimeout);
 
     virtual bool OnSendOfferSDP(
       OpalRTPSessionManager & rtpSessions,
-      SDPSessionDescription & sdpOut
+      SDPSessionDescription & sdpOut,
+      bool offerCurrentOnly
     );
     virtual bool OnSendOfferSDPSession(
       const OpalMediaType & mediaType,
@@ -644,7 +659,8 @@ class SIPConnection : public OpalRTPConnection
     );
     virtual bool OnReceivedAnswerSDPSession(
       SDPSessionDescription & sdp,
-      unsigned sessionId
+      unsigned sessionId,
+      bool & multipleFormats
     );
 
     virtual OpalMediaSession * SetUpMediaSession(
@@ -656,7 +672,7 @@ class SIPConnection : public OpalRTPConnection
     );
 
     bool SendReINVITE(PTRACE_PARAM(const char * msg));
-    void StartPendingReINVITE();
+    bool StartPendingReINVITE();
 
     friend class SIPInvite;
     static PBoolean WriteINVITE(OpalTransport & transport, void * param);
@@ -697,7 +713,8 @@ class SIPConnection : public OpalRTPConnection
     };
     HoldState             m_holdToRemote;
     bool                  m_holdFromRemote;
-    PString               forwardParty;
+    PString               m_forwardParty;
+    SIPURL                m_contactAddress;
     SIPURL                m_ciscoRemotePartyID;
 
     SIP_PDU             * originalInvite;
@@ -706,6 +723,7 @@ class SIPConnection : public OpalRTPConnection
     unsigned              m_sdpVersion; // Really a sequence number
     bool                  m_needReINVITE;
     bool                  m_handlingINVITE;
+    bool                  m_resolveMultipleFormatReINVITE;
     bool                  m_symmetricOpenStream;
     SIPDialogContext      m_dialog;
     OpalGloballyUniqueID  m_dialogNotifyId;
@@ -751,6 +769,12 @@ class SIPConnection : public OpalRTPConnection
     PSafePtr<OpalSIPIMContext> m_imContext;
 #endif
 
+    enum {
+      UserInputMethodUnknown,
+      ReceivedRFC2833,
+      ReceivedINFO
+    } m_receivedUserInputMethod;
+
   private:
     P_REMOVE_VIRTUAL_VOID(OnCreatingINVITE(SIP_PDU&));
     P_REMOVE_VIRTUAL_VOID(OnReceivedTrying(SIP_PDU &));
