@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25794 $
+ * $Revision: 28162 $
  * $Author: rjongbloed $
- * $Date: 2011-05-17 19:16:56 -0500 (Tue, 17 May 2011) $
+ * $Date: 2012-08-07 22:59:15 -0500 (Tue, 07 Aug 2012) $
  */
 
 #ifndef OPAL_SIP_SIPEP_H
@@ -393,7 +393,14 @@ class SIPEndPoint : public OpalRTPEndPoint
     bool Register(
       const SIPRegister::Params & params, ///< Registration parameters
       PString & aor,                      ///< Resultant address-of-record for unregister
-      bool asynchronous = true            ///< Wait for completion, may take some time
+      SIP_PDU::StatusCodes * reason = NULL ///< If not null, wait for completion, may take some time
+    );
+
+    // For backward compatibility
+    bool Register(
+      const SIPRegister::Params & params, ///< Registration parameters
+      PString & aor,                      ///< Resultant address-of-record for unregister
+      bool asynchronous                   ///< Wait for completion, may take some time
     );
 
     /// Registration function for backward compatibility.
@@ -667,22 +674,15 @@ class SIPEndPoint : public OpalRTPEndPoint
     );
 
 
-    /**Send SIP message
+    /**Send IM text
      */
-    virtual PBoolean Message(
-      const PURL & to, 
-      const PString & type,
-      const PString & body,
-      PURL & from, 
-      PString & conversationId
-    );
     virtual PBoolean Message(
       OpalIM & message
     );
-    OpalIMContext::SentStatus SendMESSAGE(
-      OpalIM & message
-    );
-    OpalIMContext::SentStatus SendMESSAGE(
+
+    /**Send SIP message
+     */
+    bool SendMESSAGE(
       SIPMessage::Params & params
     );
 
@@ -693,6 +693,30 @@ class SIPEndPoint : public OpalRTPEndPoint
       SIP_PDU::StatusCodes reason
     );
 
+    struct ConnectionlessMessageInfo {
+      ConnectionlessMessageInfo(OpalTransport & transport, SIP_PDU & pdu)
+        : m_pdu(pdu), m_transport(transport), m_status(ResponseSent)
+      { }
+
+      SIP_PDU & m_pdu;
+      OpalTransport & m_transport;
+      enum {
+        NotHandled,
+        SendOK,
+        MethodNotAllowed,
+        ResponseSent
+      } m_status;
+    };
+
+    typedef PNotifierTemplate<ConnectionlessMessageInfo &> ConnectionlessMessageNotifier;
+    #define PDECLARE_ConnectionlessMessageNotifier(cls, fn) PDECLARE_NOTIFIER2(SIPEndPoint, cls, fn, SIPEndPoint::ConnectionlessMessageInfo &)
+    #define PCREATE_ConnectionlessMessageNotifier(fn) PCREATE_NOTIFIER2(fn, SIPEndPoint::ConnectionlessMessageInfo &)
+
+    void SetConnectionlessMessageNotifier(
+      const ConnectionlessMessageNotifier & notifier
+    )
+    { m_onConnectionlessMessage = notifier; }
+
 
     /**Send SIP OPTIONS
      */
@@ -829,9 +853,23 @@ class SIPEndPoint : public OpalRTPEndPoint
      */
     unsigned GetNextCSeq() { return ++lastSentCSeq; }
 
+    /**Set registration search mode.
+       If true then only the user indicated as "local" address
+       (e.g. from OPAL_OPT_CALLING_PARTY_NAME etc) by the will be used
+       in searches of the registrations. No default to first user of the
+       same domain will be performed. */
+    void SetRegisteredUserMode(bool v) { m_registeredUserMode = v; }
+    bool GetRegisteredYserMode() const { return m_registeredUserMode; }
+
     /**Return the SIPAuthentication for a specific realm.
      */
-    bool GetAuthentication(const PString & authRealm, PString & user, PString & password); 
+    bool GetAuthentication(
+      SIPAuthentication & newAuthentication,
+      SIPAuthentication * oldAuthentication,
+      const SIPURL & proxy,
+      const PString & username,
+      const PString & password
+    );
     
     /**Return the registered proxy URL for the given host.
      */
@@ -859,8 +897,9 @@ class SIPEndPoint : public OpalRTPEndPoint
        the given transport is set to the Contact field.
      */
     void AdjustToRegistration(
+      SIP_PDU & pdu,
       const OpalTransport & transport,
-      SIP_PDU & pdu
+      const SIPConnection * connection
     );
 
     /**Return the local URL for the given transport and user name.
@@ -964,6 +1003,7 @@ class SIPEndPoint : public OpalRTPEndPoint
     PTimeInterval registrarTimeToLive;
     PTimeInterval notifierTimeToLive;
     PTimeInterval natBindingTimeout;
+    bool          m_registeredUserMode;
 
     bool              m_shuttingDown;
     SIPHandlersList   activeSIPHandlers;
@@ -976,8 +1016,14 @@ class SIPEndPoint : public OpalRTPEndPoint
     PAtomicInteger          lastSentCSeq;
     int                     m_defaultAppearanceCode;
 
-    std::map<PString, PSyncPoint> m_registrationComplete;
+    struct RegistrationCompletion {
+      PSyncPoint           m_sync;
+      SIP_PDU::StatusCodes m_reason;
+      RegistrationCompletion() : m_reason(SIP_PDU::Information_Trying) { }
+    };
+    std::map<PString, RegistrationCompletion> m_registrationComplete;
 
+    ConnectionlessMessageNotifier m_onConnectionlessMessage;
 
     // Thread pooling
     class SIP_Work
@@ -1032,29 +1078,6 @@ class SIPEndPoint : public OpalRTPEndPoint
     P_REMOVE_VIRTUAL_VOID(OnReceivedAuthenticationRequired(SIPTransaction &, SIP_PDU &));
     P_REMOVE_VIRTUAL_VOID(OnReceivedOK(SIPTransaction &, SIP_PDU &));
     P_REMOVE_VIRTUAL_VOID(OnMessageFailed(const SIPURL &, SIP_PDU::StatusCodes));
-
-  public:
-    struct ConnectionlessMessageInfo {
-      ConnectionlessMessageInfo(OpalTransport & transport, SIP_PDU & pdu)
-        : m_pdu(pdu), m_transport(transport), m_status(true)
-      { }
-
-      SIP_PDU & m_pdu;
-      OpalTransport & m_transport;
-      bool m_status;
-    };
-
-    typedef PNotifierTemplate<ConnectionlessMessageInfo &> ConnectionlessMessageNotifier;
-    #define PDECLARE_ConnectionlessMessageNotifier(cls, fn) PDECLARE_NOTIFIER2(SIPEndPoint, cls, fn, SIPEndPoint::ConnectionlessMessageInfo &)
-    #define PCREATE_ConnectionlessMessageNotifier(fn) PCREATE_NOTIFIER2(fn, SIPEndPoint::ConnectionlessMessageInfo &)
-
-    void SetConnectionlessMessageNotifier(
-      const ConnectionlessMessageNotifier & notifier
-    )
-    { m_onConnectionlessMessage = notifier; }
-
-  protected:
-    ConnectionlessMessageNotifier m_onConnectionlessMessage;
 };
 
 
