@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25174 $
+ * $Revision: 27644 $
  * $Author: rjongbloed $
- * $Date: 2011-02-22 21:38:08 -0600 (Tue, 22 Feb 2011) $
+ * $Date: 2012-05-16 23:42:17 -0500 (Wed, 16 May 2012) $
  */
 
 #include "precompile.h"
@@ -106,12 +106,8 @@ void CodecTest::Main()
 #endif
 
   if (args.HasOption("list")) {
-    OpalPluginCodecManager * codecMgr = PFactory<PPluginModuleManager>::CreateInstanceAs<OpalPluginCodecManager>("OpalPluginCodecManager");
-    if (codecMgr == NULL) {
-      cerr << "error: Cannot get codec manager" << endl;
-      return;
-    }
-    PPluginModuleManager::PluginListType pluginList = codecMgr->GetPluginList();
+    OpalPluginCodecManager & codecMgr = OpalPluginCodecManager::GetInstance();
+    PPluginModuleManager::PluginListType pluginList = codecMgr.GetPluginList();
 
     cout << "Plugin codecs:" << endl;
     for (int i = 0; i < pluginList.GetSize(); i++) {
@@ -121,7 +117,7 @@ void CodecTest::Main()
       PluginCodec_GetCodecFunction getCodecs;
       if (plugin.GetFunction(PLUGIN_CODEC_GET_CODEC_FN_STR, (PDynaLink::Function &)getCodecs)) {
         unsigned count = 0;
-        PluginCodec_Definition * codecs = (*getCodecs)(&count, PLUGIN_CODEC_VERSION);
+        const PluginCodec_Definition * codecs = (*getCodecs)(&count, PLUGIN_CODEC_VERSION);
         while (count > 0) {
           PString name(codecs->descr);
           if (codecNames.GetStringsIndex(name) == P_MAX_INDEX)
@@ -351,8 +347,17 @@ int TranscoderThread::InitialiseCodec(PArgList & args, const OpalMediaFormat & r
       else {
         OpalMediaFormat adjustedRawFormat = rawFormat;
         if (rawFormat == OpalPCM16) {
-          if (mediaFormat.GetClockRate() != rawFormat.GetClockRate() || mediaFormat.GetPayloadType() == RTP_DataFrame::G722)
+          if (mediaFormat.GetPayloadType() == RTP_DataFrame::G722)
             adjustedRawFormat = OpalPCM16_16KHZ;
+          else {
+            PString str = OPAL_PCM16;
+            if (mediaFormat.GetOptionInteger(OpalAudioFormat::ChannelsOption(), 1) == 2)
+              str += 'S';
+            if (mediaFormat.GetClockRate() != 8000)
+              str += '-' + PString(PString::Unsigned, mediaFormat.GetTimeUnits()) + "KHZ";
+            adjustedRawFormat = str;
+          }
+
           if (args.HasOption('F')) {
             unsigned fpp = args.GetOptionString('F').AsUnsigned();
             if (fpp > 0)
@@ -360,6 +365,24 @@ int TranscoderThread::InitialiseCodec(PArgList & args, const OpalMediaFormat & r
           }
         }
 
+        PStringArray options = args.GetOptionString('O').Lines();
+        for (PINDEX opt = 0; opt < options.GetSize(); ++opt) {
+          PString option = options[opt];
+          PINDEX equal = option.Find('=');
+          if (equal == P_MAX_INDEX)
+            cerr << "Illegal option \"" << option << "\" used." << endl;
+          else {
+            PString name = option.Left(equal);
+            PString value = option.Mid(equal+1);
+            if (mediaFormat.SetOptionValue(name, value))
+              cout << "Option \"" << name << "\" set to \"" << value << "\"." << endl;
+            else
+              cerr << "Could not set option \"" << name << "\" to \"" << value << "\"." << endl;
+          }
+        }
+
+        mediaFormat.ToCustomisedOptions();
+
         if ((m_encoder = OpalTranscoder::Create(adjustedRawFormat, mediaFormat)) == NULL) {
           cout << "Could not create encoder for media format \"" << mediaFormat << '"' << endl;
           return false;
@@ -379,27 +402,6 @@ int TranscoderThread::InitialiseCodec(PArgList & args, const OpalMediaFormat & r
 }
 
 
-void SetOptions(PArgList & args, OpalMediaFormat & mediaFormat)
-{
-  PStringArray options = args.GetOptionString('O').Lines();
-  for (PINDEX opt = 0; opt < options.GetSize(); ++opt) {
-    PString option = options[opt];
-    PINDEX equal = option.Find('=');
-    if (equal == P_MAX_INDEX)
-      cerr << "Illegal option \"" << option << "\" used." << endl;
-    else {
-      PString name = option.Left(equal);
-      PString value = option.Mid(equal+1);
-      if (mediaFormat.SetOptionValue(name, value))
-        cout << "Option \"" << name << "\" set to \"" << value << "\"." << endl;
-      else
-        cerr << "Could not set option \"" << name << "\" to \"" << value << "\"." << endl;
-    }
-  }
-
-  mediaFormat.ToCustomisedOptions();
-}
-
 static int udiff(unsigned int const subtrahend, unsigned int const subtractor) 
 {
   return subtrahend - subtractor;
@@ -514,11 +516,6 @@ bool AudioThread::Initialise(PArgList & args)
 
   if (m_encoder == NULL) 
     m_frameTime = mediaFormat.GetFrameTime();
-  else {
-    OpalMediaFormat mediaFormat = m_encoder->GetOutputFormat();
-    SetOptions(args, mediaFormat);
-    m_encoder->UpdateMediaFormats(OpalMediaFormat(), mediaFormat);
-  }
 
   cout << "opened and initialised." << endl;
   m_running = true;
@@ -716,8 +713,6 @@ bool VideoThread::Initialise(PArgList & args)
   }
   cout << "Target bit rate set to " << mediaFormat.GetOptionInteger(OpalVideoFormat::TargetBitRateOption()) << " bps" << endl;
 
-  SetOptions(args, mediaFormat);
-
   unsigned bitRate = mediaFormat.GetOptionInteger(OpalVideoFormat::TargetBitRateOption());
   PString rc = args.GetOptionString('C');
   if (rc.IsEmpty()) 
@@ -735,9 +730,12 @@ bool VideoThread::Initialise(PArgList & args)
 
   m_frameTime = mediaFormat.GetFrameTime();
   if (m_encoder != NULL) {
+    if (args.HasOption('p')) {
+      unsigned bytes = args.GetOptionString('p').AsUnsigned();
+      m_encoder->SetMaxOutputSize(bytes);
+      mediaFormat.SetOptionInteger(OpalMediaFormat::MaxTxPacketSizeOption(), bytes);
+    }
     m_encoder->UpdateMediaFormats(OpalMediaFormat(), mediaFormat);
-    if (args.HasOption('p'))
-      m_encoder->SetMaxOutputSize(args.GetOptionString('p').AsUnsigned());
   }
 
   m_singleStep = args.HasOption("single-step");
@@ -932,7 +930,7 @@ void TranscoderThread::Main()
   // main loop
   //
 
-  RTP_DataFrame * srcFrame_ = NULL;
+  RTP_DataFrame * srcFramePtr = NULL;
 
   while ((m_running && m_framesToTranscode < 0) || (m_framesToTranscode-- > 0)) {
 
@@ -940,10 +938,9 @@ void TranscoderThread::Main()
     //
     //  acquire and format source frame
     //
-    if (srcFrame_ != NULL)
-      delete srcFrame_;
-    srcFrame_ = new RTP_DataFrame(0);
-    RTP_DataFrame & srcFrame = *srcFrame_;
+    delete srcFramePtr;
+    srcFramePtr = new RTP_DataFrame(0);
+    RTP_DataFrame & srcFrame = *srcFramePtr;
 
     {
       bool state = Read(srcFrame);
@@ -983,12 +980,12 @@ void TranscoderThread::Main()
         if (isVideo) {
           if (m_forceIFrame) {
             coutMutex.Wait();
-            cout << "Decoder forced I-frame at input frame " << totalInputFrameCount-1 << endl;
+            cout << "Decoder forced I-Frame at input frame " << totalInputFrameCount-1 << endl;
             coutMutex.Signal();
           }
           if (rateControlForceIFrame) {
             coutMutex.Wait();
-            cout << "Rate controller forced I-frame at input frame " << totalInputFrameCount-1 << endl;
+            cout << "Rate controller forced I-Frame at input frame " << totalInputFrameCount-1 << endl;
             coutMutex.Signal();
           }
           if (m_forceIFrame || rateControlForceIFrame)
@@ -1052,8 +1049,8 @@ void TranscoderThread::Main()
       totalEncodedPacketCount += encFrames.GetSize();
 
       if (isVideo && m_calcSNR) {
-        ((VideoThread *)this)->SaveSNRFrame(srcFrame_);
-        srcFrame_ = NULL;
+        ((VideoThread *)this)->SaveSNRFrame(srcFramePtr);
+        srcFramePtr = NULL;
       }
 
       //////////////////////////////////////////////
@@ -1061,14 +1058,14 @@ void TranscoderThread::Main()
       //  drop encoded frames if required
       //
       if (m_dropPercent > 0) {
-        PINDEX i = 0;
-        while (i < encFrames.GetSize()) {
+        RTP_DataFrameList::iterator it = encFrames.begin();
+        while (it != encFrames.end()) {
           int n = PRandom::Number() % 100; 
           if (n >= m_dropPercent) 
-            i++;
+            ++it;
           else {
             ++totalDroppedPacketCount;
-            encFrames.RemoveAt(i);
+            encFrames.erase(it++);
           }
         }
       }
@@ -1203,6 +1200,8 @@ void TranscoderThread::Main()
     }
   }
 
+  delete srcFramePtr;
+
   //
   //  end of main loop
   //
