@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 25556 $
+ * $Revision: 27984 $
  * $Author: rjongbloed $
- * $Date: 2011-04-13 04:35:30 -0500 (Wed, 13 Apr 2011) $
+ * $Date: 2012-07-10 03:24:24 -0500 (Tue, 10 Jul 2012) $
  */
 
 #include <ptlib.h>
@@ -221,7 +221,6 @@ OpalConnection::OpalConnection(OpalCall & call,
   , displayName(ep.GetDefaultDisplayName())
   , remotePartyName(token)
   , callEndReason(NumCallEndReasons)
-  , synchronousOnRelease(true)
   , silenceDetector(NULL)
 #if OPAL_AEC
   , echoCanceler(NULL)
@@ -395,6 +394,8 @@ void OpalConnection::SetCallEndReasonText(CallEndReasonCodes reasonCode, const P
 
 void OpalConnection::SetCallEndReason(CallEndReason reason)
 {
+  PWaitAndSignal mutex(m_phaseMutex);
+
   // Only set reason if not already set to something
   if (callEndReason == NumCallEndReasons) {
     PTRACE(3, "OpalCon\tCall end reason for " << *this << " set to " << reason);
@@ -404,19 +405,25 @@ void OpalConnection::SetCallEndReason(CallEndReason reason)
 }
 
 
-void OpalConnection::ClearCall(CallEndReason reason)
+void OpalConnection::ClearCall(CallEndReason reason, PSyncPoint * sync)
 {
-  // Now set reason for the connection close
   SetCallEndReason(reason);
-  ownerCall.Clear(reason);
+  ownerCall.Clear(reason, sync);
 }
 
 
 void OpalConnection::ClearCallSynchronous(PSyncPoint * sync, CallEndReason reason)
 {
-  // Now set reason for the connection close
   SetCallEndReason(reason);
-  ownerCall.Clear(reason, sync);
+
+  PSyncPoint syncPoint;
+  if (sync == NULL)
+    sync = &syncPoint;
+
+  ClearCall(reason, sync);
+
+  PTRACE(5, "OpalCon\tSynchronous wait for " << *this);
+  sync->Wait();
 }
 
 
@@ -427,7 +434,7 @@ bool OpalConnection::TransferConnection(const PString & PTRACE_PARAM(remoteParty
 }
 
 
-void OpalConnection::Release(CallEndReason reason)
+void OpalConnection::Release(CallEndReason reason, bool synchronous)
 {
   {
     PWaitAndSignal mutex(m_phaseMutex);
@@ -436,21 +443,11 @@ void OpalConnection::Release(CallEndReason reason)
       return;
     }
     SetPhase(ReleasingPhase);
+    SetCallEndReason(reason);
   }
 
-  if (synchronousOnRelease) {
-    if (!LockReadWrite()) {
-      PTRACE(2, "OpalCon\tAlready released " << *this);
-      return;
-    }
-
-    PTRACE(3, "OpalCon\tReleasing " << *this);
-
-    // Now set reason for the connection close
-    SetCallEndReason(reason);
-
-    UnlockReadWrite();
-
+  if (synchronous) {
+    PTRACE(3, "OpalCon\tReleasing synchronously " << *this);
     OnReleased();
     return;
   }
@@ -459,21 +456,15 @@ void OpalConnection::Release(CallEndReason reason)
 
   // Add a reference for the thread we are about to start
   SafeReference();
-  PThread::Create(PCREATE_NOTIFIER(OnReleaseThreadMain), reason.AsInteger(),
+  PThread::Create(PCREATE_NOTIFIER(OnReleaseThreadMain), 0,
                   PThread::AutoDeleteThread,
                   PThread::NormalPriority,
                   "OnRelease");
 }
 
 
-void OpalConnection::OnReleaseThreadMain(PThread &, INT reason)
+void OpalConnection::OnReleaseThreadMain(PThread &, INT)
 {
-  if (LockReadWrite()) {
-    // Now set reason for the connection close
-    SetCallEndReason(CallEndReason(reason));
-    UnlockReadWrite();
-  }
-
   OnReleased();
 
   PTRACE(4, "OpalCon\tOnRelease thread completed for " << *this);
@@ -633,9 +624,17 @@ void OpalConnection::OnEstablished()
 }
 
 
-bool OpalConnection::OnTransferNotify(const PStringToString & info)
+bool OpalConnection::OnTransferNotify(const PStringToString & info,
+                                      const OpalConnection * transferringConnection)
 {
+  if (transferringConnection == this) {
+    PSafePtr<OpalConnection> otherConnection = GetOtherPartyConnection();
+    if (otherConnection != NULL)
+      otherConnection->OnTransferNotify(info, transferringConnection);
+  }
+
   return endpoint.OnTransferNotify(*this, info);
+
 }
 
 
@@ -848,18 +847,25 @@ OpalMediaStream * OpalConnection::CreateMediaStream(const OpalMediaFormat & medi
       PVideoInputDevice * videoDevice;
       PBoolean autoDeleteGrabber;
       if (CreateVideoInputDevice(mediaFormat, videoDevice, autoDeleteGrabber)) {
+        PTRACE(4, "OpalCon\tCreated capture device \"" << videoDevice->GetDeviceName() << '"');
+
         PVideoOutputDevice * previewDevice;
         PBoolean autoDeletePreview;
-        if (!CreateVideoOutputDevice(mediaFormat, PTrue, previewDevice, autoDeletePreview))
+        if (CreateVideoOutputDevice(mediaFormat, PTrue, previewDevice, autoDeletePreview))
+          PTRACE(4, "OpalCon\tCreated preview device \"" << previewDevice->GetDeviceName() << '"');
+        else
           previewDevice = NULL;
+
         return new OpalVideoMediaStream(*this, mediaFormat, sessionID, videoDevice, previewDevice, autoDeleteGrabber, autoDeletePreview);
       }
     }
     else {
       PVideoOutputDevice * videoDevice;
       PBoolean autoDelete;
-      if (CreateVideoOutputDevice(mediaFormat, PFalse, videoDevice, autoDelete))
+      if (CreateVideoOutputDevice(mediaFormat, PFalse, videoDevice, autoDelete)) {
+        PTRACE(4, "OpalCon\tCreated display device \"" << videoDevice->GetDeviceName() << '"');
         return new OpalVideoMediaStream(*this, mediaFormat, sessionID, NULL, videoDevice, false, autoDelete);
+      }
     }
   }
 
@@ -970,11 +976,7 @@ void OpalConnection::OnStopRecording(OpalMediaPatch * patch)
 
 void OpalConnection::OnPatchMediaStream(PBoolean isSource, OpalMediaPatch & patch)
 {
-  patch.SetCommandNotifier(PCREATE_NOTIFIER(OnMediaCommand), !isSource);
-
-  OpalMediaFormat mediaFormat = patch.GetSource().GetMediaFormat();
-  if (mediaFormat.IsTransportable())
-    mediaFormat = patch.GetSink()->GetMediaFormat();
+  OpalMediaFormat mediaFormat = isSource ? patch.GetSource().GetMediaFormat() : patch.GetSink()->GetMediaFormat();
 
   if (mediaFormat.GetMediaType() == OpalMediaType::Audio()) {
     if (!mediaFormat.IsTransportable()) {
@@ -999,26 +1001,17 @@ void OpalConnection::OnPatchMediaStream(PBoolean isSource, OpalMediaPatch & patc
 
 #if OPAL_PTLIB_DTMF
     if (m_detectInBandDTMF && isSource) {
-      if (mediaFormat != OpalPCM16)
-        PTRACE(3, "OpalCon\tCould not add detect DTMF filter for " << mediaFormat
-               << " on connection " << *this << ", patch " << patch);
-      else {
-        patch.AddFilter(m_dtmfDetectNotifier, OpalPCM16);
-        PTRACE(4, "OpalCon\tAdded detect DTMF filter on connection " << *this << ", patch " << patch);
-      }
+      patch.AddFilter(m_dtmfDetectNotifier, OpalPCM16);
+      PTRACE(4, "OpalCon\tAdded detect DTMF filter on connection " << *this << ", patch " << patch);
     }
 
     if (m_sendInBandDTMF && !isSource) {
-      if (mediaFormat != OpalPCM16 &&
-          mediaFormat != OpalG711_ULAW_64K &&
-          mediaFormat != OpalG711_ALAW_64K)
-        PTRACE(3, "OpalCon\tCould not add send DTMF filter for " << mediaFormat
-               << " on connection " << *this << ", patch " << patch);
-      else {
+      if (mediaFormat == OpalG711_ULAW_64K || mediaFormat == OpalG711_ALAW_64K)
         m_dtmfSendFormat = mediaFormat;
-        patch.AddFilter(m_dtmfSendNotifier, mediaFormat);
-        PTRACE(4, "OpalCon\tAdded send DTMF filter on connection " << *this << ", patch " << patch);
-      }
+      else
+        m_dtmfSendFormat = OpalPCM16;
+      patch.AddFilter(m_dtmfSendNotifier, mediaFormat);
+      PTRACE(4, "OpalCon\tAdded send DTMF filter on connection " << *this << ", patch " << patch);
     }
 #endif
   }
@@ -1153,8 +1146,7 @@ PBoolean OpalConnection::CreateVideoOutputDevice(const OpalMediaFormat & mediaFo
 
 bool OpalConnection::SendVideoUpdatePicture(unsigned sessionID, bool force) const
 {
-  PSafeLockReadWrite safeLock(*this);
-  if (!safeLock.IsLocked())
+  if (IsReleased())
     return false;
 
   OpalMediaStreamPtr stream = sessionID != 0 ? GetMediaStream(sessionID, false)
@@ -1164,11 +1156,11 @@ bool OpalConnection::SendVideoUpdatePicture(unsigned sessionID, bool force) cons
     return false;
   }
 
+  PTRACE(3, "OpalCon\tVideo update picture (I-Frame) requested in video stream " << *stream << " on " << *this);
   if (force)
     stream->ExecuteCommand(OpalVideoUpdatePicture());
   else
     stream->ExecuteCommand(OpalVideoPictureLoss());
-  PTRACE(3, "OpalCon\tUpdate video picture (I-Frame) requested in video stream " << *stream);
 
   return true;
 }
@@ -1551,6 +1543,8 @@ void OpalConnection::SetStringOptions(const StringOptions & options, bool overwr
     for (PINDEX i = 0; i < options.GetSize(); ++i)
       m_stringOptions.SetAt(options.GetKeyAt(i), options.GetDataAt(i));
   }
+
+  OnApplyStringOptions();
 }
 
 void OpalConnection::OnApplyStringOptions()
@@ -1562,6 +1556,15 @@ void OpalConnection::OnApplyStringOptions()
   if (LockReadWrite()) {
     PCaselessString str;
 
+    str = m_stringOptions(IsOriginating() ? OPAL_OPT_CALLING_PARTY_NAME : OPAL_OPT_CALLED_PARTY_NAME);
+    if (!str.IsEmpty())
+      SetLocalPartyName(str);
+
+    // Allow for explicitly haveing empty string for display name
+    str = IsOriginating() ? OPAL_OPT_CALLING_DISPLAY_NAME : OPAL_OPT_CALLED_DISPLAY_NAME;
+    if (m_stringOptions.Contains(str))
+      SetDisplayName(m_stringOptions[str]);
+
     str = m_stringOptions(OPAL_OPT_USER_INPUT_MODE);
     if (str == "RFC2833")
       SetSendUserInputMode(SendUserInputAsRFC2833);
@@ -1631,8 +1634,10 @@ OpalMediaFormatList OpalConnection::GetMediaFormats() const
 
 OpalMediaFormatList OpalConnection::GetLocalMediaFormats()
 {
-  if (m_localMediaFormats.IsEmpty())
+  if (m_localMediaFormats.IsEmpty()) {
     m_localMediaFormats = ownerCall.GetMediaFormats(*this);
+    PTRACE(4, "SIP\tLocal media formats set:\n    " << setfill(',') << m_localMediaFormats << setfill(' '));
+  }
   return m_localMediaFormats;
 }
 
@@ -1655,8 +1660,14 @@ void OpalConnection::OnStopMediaPatch(OpalMediaPatch & patch)
 }
 
 
-void OpalConnection::OnMediaCommand(OpalMediaCommand & /*command*/, INT /*extra*/)
+bool OpalConnection::OnMediaCommand(OpalMediaStream & stream, const OpalMediaCommand & command)
 {
+  PTRACE(3, "OpalCon\tOnMediaCommand \"" << command << "\" on " << stream << " for " << *this);
+  if (&stream.GetConnection() != this)
+    return false;
+
+  PSafePtr<OpalConnection> other = GetOtherPartyConnection();
+  return other != NULL && other->OnMediaCommand(stream, command);
 }
 
 
